"""
Module to create a Connection to a CDE Virtual Cluster
"""

import numpy as np
import pandas as pd
from os.path import exists
from requests_toolbelt import MultipartEncoder
import xmltodict as xd
import pyparsing
import os, json, requests, re, sys


class CdeConnection:
    """
    Class to establish a connection to a CDE Virtual Cluster
    """

    def __init__(self, JOBS_API_URL, WORKLOAD_USER, WORKLOAD_PASSWORD, TOKEN=None):
        self.JOBS_API_URL = JOBS_API_URL
        self.WORKLOAD_USER = WORKLOAD_USER
        self.WORKLOAD_PASSWORD = WORKLOAD_PASSWORD
        self.TOKEN = TOKEN
        #self.GMAIL_APP_PASSWORD = GMAIL_APP_PASSWORD

    def setToken(self):
        """
        Method to set user token to interact with CDE Service remotely
        """

        rep = self.JOBS_API_URL.split("/")[2].split(".")[0]
        GET_TOKEN_URL = self.JOBS_API_URL.replace(rep, "service").replace("dex/api/v1", "gateway/authtkn/knoxtoken/api/v1/token")

        token_json = requests.get(GET_TOKEN_URL, auth=(self.WORKLOAD_USER, self.WORKLOAD_PASSWORD))

        print("TOKEN SET SUCCESSFULLY\n")
        self.TOKEN = json.loads(token_json.text)["access_token"]



"""
Module to define CDE Resources
"""

from abc import ABC, abstractmethod

class CdeResource(ABC):
    """
    Abstract Class to define CDE Resource
    """
    @abstractmethod
    def createResourceDefinition(self):
        raise NotImplementedError

class CdeFilesResource(CdeResource):
    """
    Class to define CDE Resource of type Files
    """

    def __init__(self, CDE_RESOURCE_NAME):
        self.CDE_RESOURCE_NAME = CDE_RESOURCE_NAME

    def createResourceDefinition(self):
        """
        Method to create a CDE Resource Definition for CDE Resources of type Files
        """

        cdeFilesResourceDefinition = {
                                    "name": str(self.CDE_RESOURCE_NAME),
                                    "type":"files",
                                    "retention-policy" : "keep_indefinitely"
                                    }

        return cdeFilesResourceDefinition

class CdePythonResource(CdeResource):
    """
    Class to define CDE Resource or type Python Environment
    """

    def __init__(self, type):
        self.type = type

    def createResourceDefinition(self, CDE_RESOURCE_NAME):
        """
        Method to create a CDE Resource Definition for CDE Resources of type Python Environment
        """

        cdePythonResourceDefinition = {
                                    "name": str(CDE_RESOURCE_NAME),
                                    "type": "python-env",
                                    "python-version": "python3",
                                    "retention-policy" : "keep_indefinitely"
                                    }


        return cdePythonResourceDefinition



"""
Module to manage CDE Clusters
"""

from cdepy.cdeconnection import CdeConnection
import requests
from datetime import datetime
import pytz
import numpy as np
import pandas as pd
from os.path import exists
from requests_toolbelt import MultipartEncoder
import xmltodict as xd
import pyparsing
import os, json, requests, re, sys


class CdeClusterManager:
    """
    Class to manage CDE Clusters
    """

    def __init__(self, cdeConnection):
        self.clusterConnection = cdeConnection
        self.JOBS_API_URL = self.clusterConnection.JOBS_API_URL
        self.TOKEN = self.clusterConnection.TOKEN


    def createJob(self, cdeJobDefinition):
        """
        Method to create a CDE Job
        Requires cdeJobDefinition of type cdeAirflowJobDefinition or cdeSparkJobDefinition as input for payload
        """

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        PUT = '{}/jobs'.format(self.JOBS_API_URL)

        data = json.dumps(cdeJobDefinition)

        x = requests.post(PUT, headers=headers, data=data)

        if x.status_code == 201:
            print("CDE Job Creation Succeeded\n")
        else:
            print(x.status_code)
            print(x.text)


    def deleteJob(self, cdeJobName):
        """
        Method to delete job
        Requires cdeJobName as shown in the CDE Jobs UI
        """

        url = self.JOBS_API_URL + "/jobs/" + cdeJobName

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        x = requests.delete(url, headers=headers)

        if x.status_code == 201:
            print("CDE Job Creation Succeeded\n")
        else:
            print(x.status_code)
            print(x.text)


    def updateJob(self, cdeJobName, cdeJobDefinition):
        """
        Method to update job definition
        Requires cdeJobName as shown in the CDE Jobs UI
        Requires cdeJobDefinition of type cdeAirflowJobDefinition or cdeSparkJobDefinition as input for payload
        """

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        PUT = '{}/jobs'.format(self.JOBS_API_URL)

        data = json.dumps(cdeJobDefinition)

        x = requests.post(PUT, headers=headers, data=data)

        if x.status_code == 201:
            print("CDE Job Update Succeeded\n")
        else:
            print(x.status_code)
            print(x.text)


    def showAvailableLogTypes(self, jobRunId):
        """
        Method to show all available log types for a specific Job Run
        Job Log Types for Spark Jobs are:
        1) driver/stdout
        2) driver/stderr
        3) driver/k8sevents i.e. Pod Event Logs
        4) driver/event i.e. Spark Event Logs
        5) executor_n/stdout e.g. executor_1/stdout, executor_2/stdout
        6) driver/tgtloader
        7) driver/workspace-init
        8) submitter/stderr
        9) submitter/stdout
        10) submitter/k8s
        11) submitter/jobs_api
        """

        url = self.JOBS_API_URL + "/job-runs/" + jobRunId + "/log-types"

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        x = requests.get(url, headers=headers)

        return x.text

        if x.status_code == 201:
            print("Downloading Job Logs Succeeded")
        else:
            print(x.status_code)
            print(x.text)


    def downloadJobRunLogs(self, jobRunId, logsType):
        """
        Method to download all logs for specified jobrun
        Requires jobRunId; jobRunId is an integer; jobRunId can be obtained by running listJobRuns
        """

        url = self.JOBS_API_URL + "/job-runs/" + jobRunId + "/logs?type=" + logsType

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        x = requests.get(url, headers=headers)

        return x.text

        if x.status_code == 201:
            print("Downloading Job Logs Succeeded")
        else:
            print(x.status_code)
            print(x.text)


    def listJobs(self):
        """
        Method to list CDE Jobs as shown in the CDE Jobs UI
        Shows all jobs in the CDE Virtual Cluster
        """

        url = self.JOBS_API_URL + "/jobs"

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        x = requests.get(url+'?latestjob=false&limit=100&offset=0&orderby=name&orderasc=true', headers=headers)

        return x.text

        if x.status_code == 201:
            print("Listing Jobs Succeeded")
        else:
            print(x.status_code)
            print(x.text)


    def listJobRuns(self, jobType=None):
        """
        Method to show all CDE Jobs that have been executed in the cluster
        Does not require input
        """

        tz_LA = pytz.timezone('America/Los_Angeles')
        now = datetime.now(tz_LA)
        print("Listing Jobs as of: {} PACIFIC STANDARD TIME\n".format(now))


        url = self.JOBS_API_URL + "/job-runs"

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }
        if jobType=="spark":
            x = requests.get(url+'?filter=type%5Beq%5Dspark&limit=100&offset=0&orderby=ID&orderasc=false', headers=headers)
        elif jobType=="airflow":
            x = requests.get(url+'?filter=type%5Beq%5Dairflow&limit=100&offset=0&orderby=ID&orderasc=false', headers=headers)
        else:
            x = requests.get(url+'?limit=100&offset=0&orderby=ID&orderasc=false', headers=headers)

        return x.text

        if x.status_code == 201:
            print("Listing Jobs Succeeded")
        else:
            print(x.status_code)
            print(x.text)


    def runJob(self, CDE_JOB_NAME, SPARK_OVERRIDES=None, AIRFLOW_OVERRIDES=None):
        """
        Method to trigger execution of CDE Job
        CDE Job could be of type Spark or Airflow
        The method assumes the CDE Job has already been created in the CDE Virtual Cluster
        """
        overrides = {}

        if SPARK_OVERRIDES != None and AIRFLOW_OVERRIDES != None:
            print("Error: Spark Overrides and Airflow Overrides Specified\n")
            print("You can only specify either Spark Overrides or Airflow Overrides, but not both!")
        elif SPARK_OVERRIDES != None and AIRFLOW_OVERRIDES == None and isinstance(SPARK_OVERRIDES, dict):
            overrides = {"overrides": SPARK_OVERRIDES}
        elif AIRFLOW_OVERRIDES != None and SPARK_OVERRIDES == None and isinstance(AIRFLOW_OVERRIDES, dict):
            overrides = {"overrides": AIRFLOW_OVERRIDES}

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        POST = "{}/jobs/".format(self.JOBS_API_URL)+CDE_JOB_NAME+"/run"

        data = json.dumps(overrides)

        x = requests.post(POST, headers=headers, data=data)

        if x.status_code == 201:
            print("CDE Job Submission has Succeeded\n")
            print("Please visit the CDE Job Runs UI to validate CDE Job Status\n")
        else:
            print(x.status_code)
            print(x.text)


    def createResource(self, cdeRsourceDefinition):
        """
        Method to create CDE Resource
        Requires cdeRsourceDefinition as input
        Accepts types Files or Python
        e.g. cdeRsourceDefinition = {"name": str(resource_name)}
        """

        print("CDE Resource Creation in Progress\n")

        url = self.JOBS_API_URL + "/resources"
        data_to_send = json.dumps(cdeRsourceDefinition).encode("utf-8")

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        x = requests.post(url, data=data_to_send, headers=headers)

        if x.status_code == 201:
            print("CDE Resource Created Successfully")
        else:
            print(x.status_code)
            print(x.text)


    def deleteResource(self, cdeResourceName):
        """
        Method to delete CDE Resource
        Requires cdeRsourceDefinition name as input
        e.g. cdeResourceName = str(resource_name)
        """

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        url = self.JOBS_API_URL + "/resources/" + cdeResourceName

        x = requests.delete(url, headers=headers)

        if x.status_code == 204:
            print("CDE Resource {} Deleted Successfully\n".format(cdeResourceName))
        else:
            print(x.status_code)
            print(x.text)


    def describeResource(self, CDE_RESOURCE_NAME, includeFiles = True):
        """
        Method to get resource configuration and content
        """

        if includeFiles == True:
            url = self.JOBS_API_URL + "/resources/" + CDE_RESOURCE_NAME + "?includeFiles=true"
        elif includeFiles == False:
            url = self.JOBS_API_URL + "/resources/" + CDE_RESOURCE_NAME + "?includeFiles=false"
        else:
            print("Error: Include Files Flag can only be set to True or False")

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        x = requests.get(url, headers=headers)

        return x.text

        if x.status_code == 201:
            print("Downloading Job Logs Succeeded")
        else:
            print(x.status_code)
            print(x.text)


    #Upload Spark CDE Job file to CDE Resource
    def uploadFileToResource(self, CDE_RESOURCE_NAME, LOCAL_FILE_PATH, LOCAL_FILE_NAME):
        """
        Method to uplaod files from local to CDE Resource
        Can be used to:
            1) upload files to a CDE Resource of type Files
            2) uplaod a "requirements.txt" file to a CDE Resource of type Python Environment
        Requires a CDE_RESOURCE_NAME, LOCAL_FILE_PATH and LOCAL_FILE_NAME
        e.g. "myCdeFilesResource", "~/myfiles/cdefiles", and "mySparkScript.py"
        e.g. "myCdePythonResource", "~/myfiles/cdefiles", and "requirements.txt"
        """

        print("Uploading File {0} to CDE Resource {1}\n".format(LOCAL_FILE_NAME, CDE_RESOURCE_NAME))

        m = MultipartEncoder(
            fields={
                    'file': ('filename', open(LOCAL_FILE_PATH+"/"+LOCAL_FILE_NAME, 'rb'), 'text/plain')}
            )

        PUT = '{jobs_api_url}/resources/{resource_name}/{file_name}'.format(jobs_api_url=self.JOBS_API_URL, resource_name=CDE_RESOURCE_NAME, file_name=LOCAL_FILE_NAME)

        x = requests.put(PUT, data=m, headers={'Authorization': f"Bearer {self.TOKEN}",'Content-Type': m.content_type})
        print("Response Status Code {}".format(x.status_code))

        if x.status_code == 201:
            print("Uploading File {0} to CDE Resource {1} has Succeeded\n".format(LOCAL_FILE_NAME, CDE_RESOURCE_NAME))
        else:
            print(x.status_code)
            print(x.text)


    def uploadArchiveToResource(self, CDE_RESOURCE_NAME, LOCAL_FILE_PATH, LOCAL_FILE_NAME):
        """
        Method to Upload an archive(.zip or .tar.gz archives)
        to the resource with an optional directory prefix.
        New files are added and existing files are overwritten
        """

        raise NotImplementedError


    def removeFileFromResource(self, CDE_RESOURCE_NAME, RESOURCE_FILE_NAME):
        """
        Method to remove a file from a CDE Resource of type Files
        """

        url = self.JOBS_API_URL + "/resources/" + CDE_RESOURCE_NAME + "/" + RESOURCE_FILE_NAME

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        x = requests.delete(url, headers=headers)

        if x.status_code == 201:
            print("File Removal Succeeded\n")
        else:
            print(x.status_code)
            print(x.text)


    def downloadFileFromResource(self, CDE_RESOURCE_NAME, RESOURCE_FILE_NAME):
        """
        Method to download a file in the resource at the path specified
        """

        url = self.JOBS_API_URL + "/resources/" + CDE_RESOURCE_NAME + "/" + RESOURCE_FILE_NAME

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/octet-stream',
            'Content-Type': 'application/json',
        }

        x = requests.get(url, headers=headers)

        return x.text

        if x.status_code == 201:
            print("Downloading Resource File Succeeded")
        else:
            print(x.status_code)
            print(x.text)


    def pauseAllJobs(self):
        """
        Method to pause all jobs in a CDE Virtual Cluster
        """
        raise NotImplementedError


    def unpauseAllJobs(self):
        """
        Method to unpuase all paused jobs in a CDE Virtual Cluster
        """
        raise NotImplementedError


    def pauseSingleJob(self, CDE_JOB_NAME):
        """
        Method to pause a single unpaused job
        """

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        POST = "{}/jobs/".format(self.JOBS_API_URL) + CDE_JOB_NAME + "/schedule/pause"

        x = requests.post(POST, headers=headers)

        if x.status_code == 201:
            print("CDE Job Submission has Succeeded\n")
            print("Please visit the CDE Job Runs UI to validate CDE Job Status\n")
        else:
            print(x.status_code)
            print(x.text)


    def unpauseSingleJob(self, CDE_JOB_NAME):
        """
        Method to unpuase a single paused job
        """

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
        }

        POST = "{}/jobs/".format(self.JOBS_API_URL) + CDE_JOB_NAME + "/schedule/unpause"

        x = requests.post(POST, headers=headers)

        if x.status_code == 201:
            print("CDE Job Submission has Succeeded\n")
            print("Please visit the CDE Job Runs UI to validate CDE Job Status\n")
        else:
            print(x.status_code)
            print(x.text)


    def listVcMeta(self):
        """
        Method to provide configuration information
        and useful parameters for the CDE Virtual Cluster
        """

        headers = {
            'Authorization': f"Bearer {self.TOKEN}",
            'accept': 'application/json',
            'Content-Type': 'application/json',
            }

        x = requests.get(self.JOBS_API_URL+'/info', headers=headers)

        return json.loads(x.text)



"""
Class to manage CDE Credentials
"""

import numpy as np
import pandas as pd
from os.path import exists
from requests_toolbelt import MultipartEncoder
import xmltodict as xd
import pyparsing
import os, json, requests, re, sys
from cdepy.cdeconnection import CdeConnection

class CdeCredentialsManager():
  """
  Class to manage CDE Credentials
  """

  def __init__(self, cdeConnection):
      self.clusterConnection = cdeConnection
      self.JOBS_API_URL = self.clusterConnection.JOBS_API_URL
      self.TOKEN = self.clusterConnection.TOKEN


  def listCredentials(self):
    """
    Method to list all credentials
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
        'Content-Type': 'application/json',
    }

    params = (
        ('limit', '100'),
        ('offset', '0'),
        ('orderby', 'name'),
        ('orderasc', 'true'),
    )

    x = requests.get('{}/credentials'.format(self.JOBS_API_URL), headers=headers, params=params)

    return x.text

    if x.status_code == 201:
        print("Listing CDE Repositories has Succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def createBasicCredential(self, credentialName, credentialUsername, credentialPassword):
    """
    Method to create a credential
    """

    repoDefinition = {
      "basic": {
        "username": credentialUsername,
        "password": credentialPassword
      },
      "name": credentialName,
      "skipCredentialValidation": True,
      "type": "basic"
    }

    data = json.dumps(repoDefinition)

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
        'Content-Type': 'application/json',
    }

    x = requests.post('{}/credentials'.format(self.JOBS_API_URL), headers=headers, data=data)

    if x.status_code == 201:
        print("CDE Credential Creation has Succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def describeCredential(self, credentialName):
    """
    Method to describe a credential
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
        'Content-Type': 'application/json',
    }

    params = (
        ('includeFiles', 'true'),
    )

    x = requests.get('{0}/credentials/{1}'.format(self.JOBS_API_URL, credentialName), headers=headers, params=params)

    if x.status_code == 201:
        print("CDE Credential Description has Succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def deleteCredential(self, credentialName):
    """
    Method to delete a repository in CDE
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
        'Content-Type': 'application/json',
    }

    x = requests.delete('{0}/credentials/{1}'.format(self.JOBS_API_URL, credentialName), headers=headers)

    if x.status_code == 201:
        print("CDE Credential Deletion has Succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def updateBasicCredential(self, credentialName, credentialUsername, credentialPassword):
    """
    Method to update a basic CDE Credential
    """
    raise NotImplementedError



"""
Module to create a Connection to a CDE Virtual Cluster
"""

import numpy as np
import pandas as pd
from os.path import exists
from requests_toolbelt import MultipartEncoder
import xmltodict as xd
import pyparsing
import os, json, requests, re, sys
from cdepy.cdeconnection import CdeConnection

class CdeAirflowPythonEnv():
  """
  Class to manage Airflow Python Env maintenance sessions
  """

  def __init__(self, cdeConnection):
      self.clusterConnection = cdeConnection
      self.JOBS_API_URL = self.clusterConnection.JOBS_API_URL
      self.TOKEN = self.clusterConnection.TOKEN


  def createMaintenanceSession(self):
    """
    Method to create a Maintenance Session to Manage CDE Airflow Python Environments
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
        'Content-Type': 'application/json',
    }

    POST = "{}/admin/airflow/env/maintenance".format(self.JOBS_API_URL)

    x = requests.post(POST, headers=headers)

    if x.status_code == 201:
        print("CDE Maintenance Session Creation has Succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def createRepositoryPayload(self):
    """
    Method to create the Payload for the createRepository method
    """
    raise NotImplementedError("Can't use thie method yet!")


  def createPipRepository(self):
    """
    Method to create a pip repository
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
        'Content-Type': 'application/json',
    }

    POST = "{}/admin/airflow/env/maintenance/repos".format(self.JOBS_API_URL)

    payloadData = {"pipRepository": {"url": "https://pypi.org/simple/"}}
    #"skipCertValidation": true,

    data = json.dumps(payloadData)

    x = requests.post(POST, headers=headers, data=data)

    if x.status_code == 201:
        print("CDE pip repository creation has succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def checkAirflowPythonEnvStatus(self):
    """
    Method to check status of Airflow Python Environment
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
    }

    x = requests.get('{}/admin/airflow/env/maintenance/status'.format(self.JOBS_API_URL), headers=headers)

    if x.status_code == 201:
        print("CDE Airflow Python Environment Status Check has Succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def buildAirflowPythonEnv(self, pathToRequirementsTxt):
    """
    Method to build the airflow python environment with the provided requirements.txt file
    """

    m = MultipartEncoder(
        fields={
                'file': ('filename', open(pathToRequirementsTxt, 'rb'), 'text/plain')}
        )

    x = requests.post('{}/admin/airflow/env/maintenance/build'.format(self.JOBS_API_URL), data=m,
                      headers={'Authorization': f"Bearer {self.TOKEN}",'Content-Type': m.content_type})

    if x.status_code == 201:
        print("CDE Airflow Python Env Build has succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def activateAirflowPythonEnv(self):
    """
    Method to activate Python Airflow Environment
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
    }

    x = requests.post('{}/admin/airflow/env/maintenance/activate'.format(self.JOBS_API_URL), headers=headers)

    if x.status_code == 201:
        print("CDE Airflow Python Env Activation has succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def deleteAirflowPythonEnv(self):
    """
    Delete Airflow Python Environment
    """

    headers = {
        'accept': 'application/json',
    }

    x = requests.delete('{}/admin/airflow/env'.format(self.JOBS_API_URL), headers=headers)

    if x.status_code == 201:
        print("CDE Airflow Python Env Deletion has succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def deleteMaintenanceSession(self):
    """
    Method to cancel the current Maintenance Session to Manage Airflow Python Environment
    """

    headers = {
        'accept': 'application/json',
    }

    x = requests.delete('{}/admin/airflow/env/maintenance'.format(self.JOBS_API_URL), headers=headers)

    if x.status_code == 201:
        print("CDE Airflow Python Env Deletion has succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def viewMaintenanceSessionLogs(self):
    """
    Method to obtain logs for a Maintenance Session
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
    }

    x = requests.get('{}/admin/airflow/env/maintenance/logs'.format(self.JOBS_API_URL), headers=headers)

    if x.status_code == 201:
        print("CDE Airflow Python Env Deletion has succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def viewEnvironmentLogs(self):
    """
    Method to obtain Environment Logs
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
    }

    x = requests.get('{}/admin/airflow/env/logs'.format(self.JOBS_API_URL), headers=headers)

    if x.status_code == 201:
        print("CDE Airflow Python Env Deletion has succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def getAirflowPythonEnvironmentDetails(self):
    """
    Method to obtain description of airflow python env that is currently handled in the maintenance operation.
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
    }

    x = requests.get('{}/admin/airflow/env'.format(self.JOBS_API_URL), headers=headers)

    if x.status_code == 201:
        print("Current Airflow Python Env Details Request has succeeded\n")
    else:
        print(x.status_code)
        print(x.text)



"""
Class to manage CDE Repositories
"""

import numpy as np
import pandas as pd
from os.path import exists
from requests_toolbelt import MultipartEncoder
import xmltodict as xd
import pyparsing
import os, json, requests, re, sys
from cdepy.cdeconnection import CdeConnection

class CdeRepositoryManager():
  """
  Class to manage CDE Repositories
  """

  def __init__(self, cdeConnection):
      self.clusterConnection = cdeConnection
      self.JOBS_API_URL = self.clusterConnection.JOBS_API_URL
      self.TOKEN = self.clusterConnection.TOKEN


  def listRepositories(self):
    """
    Method to list all repositories
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
        'Content-Type': 'application/json',
    }

    params = (
        ('includeFiles', 'true'),
        ('limit', '100'),
        ('offset', '0'),
        ('orderby', 'name'),
        ('orderasc', 'true'),
    )

    x = requests.get('{}/repositories'.format(self.JOBS_API_URL), headers=headers, params=params)

    return x.text

    if x.status_code == 201:
        print("Listing CDE Repositories has Succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def createRepository(self, repoName, repoPath, repoCredentialName=None, repoBranch="main"):
    """
    Method to create a repository
    """

    repoDefinition = {
      "git": {
        "branch": repoBranch,
        "repository": repoPath,
        "credential": repoCredentialName
      },
      "name": repoName,
      "skipCredentialValidation": True
    }

    data = json.dumps(repoDefinition)

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
        'Content-Type': 'application/json',
    }

    x = requests.post('{}/repositories'.format(self.JOBS_API_URL), headers=headers, data=data)

    if x.status_code == 201:
        print("CDE Repository Creation has Succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def describeRepository(self, repoName):
    """
    Method to describe a repository
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
        'Content-Type': 'application/json',
    }

    params = (
        ('includeFiles', 'true'),
    )

    x = requests.get('{0}/repositories/{1}'.format(self.JOBS_API_URL, repoName), headers=headers, params=params)
    return x.text

    if x.status_code == 201:
        print("CDE Repository Description has Succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def pullRepository(self, repoName):
    """
    Method to pull latest commit from repository
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
        'Content-Type': 'application/json',
    }

    x = requests.post('{0}/repositories/{1}'.format(self.JOBS_API_URL, repoName), headers=headers)

    if x.status_code == 201:
        print("CDE Repository Description has Succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def deleteRepository(self, repoName):
    """
    Method to delete a repository in CDE
    """

    headers = {
        'Authorization': f"Bearer {self.TOKEN}",
        'accept': 'application/json',
        'Content-Type': 'application/json',
    }

    x = requests.delete('{0}/repositories/{1}'.format(self.JOBS_API_URL, repoName), headers=headers)

    if x.status_code == 201:
        print("CDE Repository Description has Succeeded\n")
    else:
        print(x.status_code)
        print(x.text)


  def downloadFileFromRepo(self, repoName, filePath):
    """
    Method to download a file from a CDE repository
    """

    headers = {
    'Authorization': f"Bearer {self.TOKEN}",
    'accept': 'application/octet-stream',
    'Content-Type': 'application/json'
    }

    x = requests.get('{}/repositories/{}/{}'.format(self.JOBS_API_URL, repoName, filePath), headers=headers)

    return x.content

    if x.status_code == 201:
        print("CDE Repository Description has Succeeded\n")
    else:
        print(x.status_code)
        print(x.text)

  def syncRepository(self, repoName):
    """
    Method to sync a cde repo with its git origin
    """

    headers = {
    'Authorization': f"Bearer {self.TOKEN}",
    'accept': 'application/octet-stream',
    'Content-Type': 'application/json'
    }

    x = requests.post('{}/repositories/{}'.format(self.JOBS_API_URL, repoName), headers=headers)

    return x.content

    if x.status_code == 201:
        print("CDE Repository Sync with Git Origin has Succeeded\n")
    else:
        print(x.status_code)
        print(x.text)



"""
Module to define CDE Jobs
"""

from abc import ABC, abstractmethod
from cdepy.cdeconnection import CdeConnection

class CdeJob(ABC):
    """
    Class to define CDE Job
    """
    @abstractmethod
    def createJobDefinition(self):
        pass

    @abstractmethod
    def hasKey(self):
        pass


class CdeSparkJob(CdeJob):
    """
    Class to define CDE Spark Jobs
    """

    def __init__(self, cdeConnection):
        self.clusterConnection = cdeConnection
        self.WORKLOAD_USER = self.clusterConnection.WORKLOAD_USER

    def hasKey(self, d, key, input_val, cdeSparkJobDefinition):
        try:
            if key in d:
                val = d[key]
                print("Spark Conf {} is found".format(key))
                if val == None:
                    cdeSparkJobDefinition[key] = input_val
                    print("Spark Conf {} is added".format(key))
                    return cdeSparkJobDefinition
                else:
                    cdeSparkJobDefinition[val][key] = input_val
                    print("Spark Conf {} is added".format(key))
                    return cdeSparkJobDefinition
        except:
            print("\nError Processing Option: ", key)
        else:
            print("\nProvided Option Not Supported: ", key)
            print("\nPlease try again.")

    def createJobDefinition(self, CDE_JOB_NAME, CDE_RESOURCE_NAME, APPLICATION_FILE_NAME, SPARK_CONFS={"spark.pyspark.python": "python3"}, **kwargs):
        """
        Method to create CDE Spark Job Definition
        Requires CDE Job Name, CDE Files Resource Name,
        Application File Name, and optionally spark configs
        """

        cdeSparkJobDefinition = {
              "name": CDE_JOB_NAME, #CDE Job Name As you want it to appear in the CDE JOBS UI
              "type": "spark",
              "retentionPolicy": "keep_indefinitely",
              "mounts": [
                {
                  "resourceName": CDE_RESOURCE_NAME
                }
              ],
              "spark": {
                "logLevel": "INFO",
                "file": APPLICATION_FILE_NAME
              },
              "schedule": {
                "enabled": False
              }
            }

        sparkConfigs = {
            "alertAfterDuration": "alerting",
            "emailOnFailure": "alerting",
            "emailOnSLAMiss": "alerting",
            "mailTo": "alerting",
            "dataConnectors": None,
            "dirPrefix":"mounts",
            "resourceName": "mounts",
            "name": "string",
            "retentionPolicy": None,
            "runtimeImageResourceName": None,
            "catchup": "schedule",
            "cronExpression": "schedule",
            "dependsOnPast": "schedule",
            "enabled": "schedule",
            "end": "schedule",
            "nextExecution": "schedule",
            "paused": "schedule",
            "pausedUponCreation": "schedule",
            "start": "schedule",
            "user": "schedule",
            "args": "spark",
            "className": "spark",
            "driverCores": "spark",
            "driverMemory": "spark",
            "executorCores": "spark",
            "executorMemory": "spark",
            "files": "spark",
            "jars": "spark",
            "logLevel": "spark",
            "name": "spark",
            "numExecutors": "spark",
            "proxyUser": "spark",
            "pyFiles": "spark",
            "pythonEnvResourceName": "spark",
            "type": "spark",
            "workloadCredentials":"spark"}

        for param in kwargs:
            print(param)
            cdeSparkJobDefinition = self.hasKey(sparkConfigs, param, kwargs[param], cdeSparkJobDefinition)

        return cdeSparkJobDefinition


class CdeAirflowJob(CdeJob):
    """
    Class to define CDE Airflow Jobs
    """

    def __init__(self, cdeConnection):
        self.clusterConnection = cdeConnection
        self.WORKLOAD_USER = self.clusterConnection.WORKLOAD_USER

    def hasKey(d, key, input_val, cdeAirflowJobDefinition):
        try:
            if key in d:
                val = d[key]
                if val == None:
                    cdeAirflowJobDefinition[key] = input_val
                    return cdeAirflowJobDefinition
                else:
                    cdeAirflowJobDefinition[val][key] = input_val
                    return cdeAirflowJobDefinition
        except:
            print("Error Processing Option: ", key)
        else:
            print("Provided Option Not Supported: ", key)

    def createJobDefinition(self, CDE_JOB_NAME, DAG_FILE, CDE_RESOURCE_NAME, AIRFLOW_FILE_MOUNTS=None):
        """
        Method to create CDE Job Definition of type Airflow
        Requires CDE Job Name, Application File Name and optionally CDE Files Resource Name
        """

        cdeAirflowJobDefinition = {
              "name": CDE_JOB_NAME,# CDE Job Name As you want it to appear in the CDE JOBS UI
              "type": "airflow",
              "hidden": True,
              "retentionPolicy": "keep_indefinitely",
              "airflow": {
                "dagFile": DAG_FILE
                      },
               "mounts": [
                 {
                  "dirPrefix": "/",
                  "resourceName": CDE_RESOURCE_NAME
                }
              ]

            }

        if AIRFLOW_FILE_MOUNTS != None and isinstance(AIRFLOW_FILE_MOUNTS, list):
            cdeAirflowJobDefinition['fileMounts'] = AIRFLOW_FILE_MOUNTS

        return cdeAirflowJobDefinition



from setuptools import setup, find_packages
import os

def read_file(filename):
    with open(os.path.join(os.path.dirname(__file__), filename)) as file:
        return file.read()

setup(
    author="Paul de Fusco",
    description="A Python Package for interacting with Cloudera Data Engineering Clusters",
    long_description=read_file("README.md"),
    long_description_content_type="text/markdown",
    name="cdepy",
    version="0.1.19",
    packages=find_packages(include=["cdepy", "cdepy.*"]),
    install_requires=["pyparsing==3.0.9", "requests-toolbelt==1.0.0", "xmltodict==0.13.0"],
    python_requires=">=2.7",
    license="MIT",
    url = "https://github.com/pdefusco/cdepy",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
  ]
)



"""
Helper Methods
"""
import requests
import json

def sparkEventLogParser(sparkLogs):
      """
      Method to reformat CDE Spark Event Logs
      Removes unwanted characters from the provided Spark Event Logs
      """

      cleanLogs = sparkLogs.replace("\n", "")

      return cleanLogs

def myUtil():
    raise NotImplementedError

def listAvailableConfs(jobType="Spark"):
    """
    Method to list available Spark or Airflow Configurations
    that user can choose from
    when creating a CDE Spark Job Definition
    with createJobDefinition method of cdejob module
    """

    if jobType=="Spark":

        sparkConfigs = {
            "alertAfterDuration": "alerting",
            "emailOnFailure": "alerting",
            "emailOnSLAMiss": "alerting",
            "mailTo": "alerting",
            "dataConnectors": None,
            "dirPrefix":"mounts",
            "resourceName": "mounts",
            "name": "string",
            "retentionPolicy": None,
            "runtimeImageResourceName": None,
            "catchup": "schedule",
            "cronExpression": "schedule",
            "dependsOnPast": "schedule",
            "enabled": "schedule",
            "end": "schedule",
            "nextExecution": "schedule",
            "paused": "schedule",
            "pausedUponCreation": "schedule",
            "start": "schedule",
            "user": "schedule",
            "args": "spark",
            "className": "spark",
            "driverCores": "spark",
            "driverMemory": "spark",
            "executorCores": "spark",
            "executorMemory": "spark",
            "files": "spark",
            "jars": "spark",
            "logLevel": "spark",
            "name": "spark",
            "numExecutors": "spark",
            "proxyUser": "spark",
            "pyFiles": "spark",
            "pythonEnvResourceName": "spark",
            "type": "spark",
            "workloadCredentials":"spark"}

        print(sparkConfigs.keys())

    elif jobType == "Airflow":
        pass

    else:
        print("Error. Please enter jobType of either 'Airflow' or 'Spark'")
